@@ -4,7 +4,7 @@ from invitations.app_settings import app_settings as invitations_settings
 from invitations.signals import invite_accepted
 from xlibrary.viewsets import ModelViewset as xViewSet
 from rest_framework import mixins, status, viewsets
-from rest_framework.decorators import (action,api_view,
+from rest_framework.decorators import (action, api_view,
                                        permission_classes)
 from rest_framework.permissions import AllowAny, IsAuthenticated
 from rest_framework.response import Response
@@ -12,17 +12,22 @@ from .models import GuestInvitation as InvitationModel, GroupOptions, User
 from django.contrib.auth.models import Group, Permission
 from django.contrib.contenttypes.models import ContentType
 # from .serializers import InvitationReadSerializer,InvitationBulkWriteSerializer,InvitationWriteSerializer,UserSerializer, GroupSerializer, PermissionSerializer,ContenttypeSerializer,GroupOptionsSerializer,CUserSerializer, CGroupSerializer,CGroupOptionsSerializer
-from .serializers import InvitationReadSerializer,InvitationBulkWriteSerializer,InvitationWriteSerializer,UserSerializer, GroupSerializer, PermissionSerializer,ContenttypeSerializer,GroupOptionsSerializer,CUserSerializer, CGroupSerializer
+from .serializers import InvitationReadSerializer, InvitationBulkWriteSerializer, InvitationWriteSerializer, UserSerializer, GroupSerializer, PermissionSerializer, ContenttypeSerializer, GroupOptionsSerializer, CUserSerializer, CGroupSerializer
 from rest_auth.views import LoginView as RestAuthLoginView
 from django.conf import settings
-AuthorModel = getattr(settings, 'INVITATIONS_AUTHOR_MODEL', '')
+from .utils import get_author_model
+from django.db.models.functions import Concat
+AuthorModel = get_author_model()
+
+
 def getVisiblePermissions():
     ct = ContentType.objects.all()
     myNameList = []
     for c in ct:
         class_name = c.model_class()
         if hasattr(class_name, "xtrmMeta"):
-            myNameList.append(class_name.__name__.lower())
+            myNameList.append((class_name._meta.app_label +
+                               class_name.__name__).lower())
     return myNameList
 
 
@@ -31,8 +36,9 @@ def jwt_response_payload_handler(token, user=None, request=None):
     perms = user.groups.first()
     rtn = []
     if perms:
-        for perm in perms.permissions.all().filter(content_type__model__in=myNameList):
+        for perm in perms.permissions.all().annotate(num_authors=Concat('content_type__app_label', 'content_type__model')).filter(num_authors__in=myNameList):
             pm = {
+                "id": perm.id,
                 "name": perm.name,
                 "codename": perm.codename,
                 "app_label": perm.content_type.app_label,
@@ -76,11 +82,11 @@ class LoginView(RestAuthLoginView):
 
 
 class UserViewSet(xViewSet):
-    http_method_names = [m for m in viewsets.ModelViewSet.http_method_names if m not in [m for m in viewsets.ModelViewSet.http_method_names if m not in ['get']]]
+    #http_method_names = [m for m in viewsets.ModelViewSet.#http_method_names if m not in [m for m in viewsets.ModelViewSet.#http_method_names if m not in ['get']]]
     queryset = User.objects.all()
     serializer_class = UserSerializer
     reporttitle = 'Users'
-	#orientation='landscape'
+#orientation='landscape'
     options = {'canAdd': 1, 'canPrint': 1}
     filters = [
         {'title': 'User Name', 'name': 'filter{username}', 'url': 'v1/rights/users/'},
@@ -98,7 +104,7 @@ class UserViewSet(xViewSet):
 
 
 class GroupViewSet(xViewSet):
-    http_method_names = [m for m in viewsets.ModelViewSet.http_method_names if m not in [m for m in viewsets.ModelViewSet.http_method_names if m not in ['get']]]
+    #http_method_names = [m for m in viewsets.ModelViewSet.#http_method_names if m not in [m for m in viewsets.ModelViewSet.#http_method_names if m not in ['get']]]
     queryset = Group.objects.all()
     serializer_class = GroupSerializer
     reporttitle = 'Groups'
@@ -115,7 +121,7 @@ class GroupViewSet(xViewSet):
 
 
 class PermissionViewSet(xViewSet):
-    http_method_names = [m for m in viewsets.ModelViewSet.http_method_names if m not in [m for m in viewsets.ModelViewSet.http_method_names if m not in ['get']]]
+    #http_method_names = [m for m in viewsets.ModelViewSet.#http_method_names if m not in [m for m in viewsets.ModelViewSet.#http_method_names if m not in ['get']]]
     queryset = Permission.objects.all()
     serializer_class = PermissionSerializer
     reporttitle = 'Permissions'
@@ -137,7 +143,7 @@ class PermissionViewSet(xViewSet):
 
 
 class ContenttypeViewSet(xViewSet):
-    http_method_names = [m for m in viewsets.ModelViewSet.http_method_names if m not in [m for m in viewsets.ModelViewSet.http_method_names if m not in ['get']]]
+    #http_method_names = [m for m in viewsets.ModelViewSet.#http_method_names if m not in [m for m in viewsets.ModelViewSet.#http_method_names if m not in ['get']]]
     queryset = ContentType.objects.all()
     serializer_class = ContenttypeSerializer
     reporttitle = 'Content Types'
@@ -156,13 +162,13 @@ class ContenttypeViewSet(xViewSet):
 
 
 class GroupOptionsViewSet(xViewSet):
-    http_method_names = [m for m in viewsets.ModelViewSet.http_method_names if m not in [m for m in viewsets.ModelViewSet.http_method_names if m not in ['get']]]
+    #http_method_names = [m for m in viewsets.ModelViewSet.#http_method_names if m not in [m for m in viewsets.ModelViewSet.#http_method_names if m not in ['get']]]
     queryset = GroupOptions.objects.all()
     serializer_class = GroupOptionsSerializer
 
 
 class CUserViewSet(viewsets.ModelViewSet):
-    http_method_names = [m for m in viewsets.ModelViewSet.http_method_names if m not in ['get']]
+    #http_method_names = [m for m in viewsets.ModelViewSet.#http_method_names if m not in ['get']]
     queryset = User.objects.all()
     serializer_class = CUserSerializer
 
@@ -172,7 +178,7 @@ class CUserViewSet(viewsets.ModelViewSet):
 
 
 class CGroupViewSet(viewsets.ModelViewSet):
-    http_method_names = ['post', 'put', 'patch']
+    #http_method_names = ['post', 'put', 'patch']
     queryset = Group.objects.all()
     serializer_class = CGroupSerializer
 
@@ -212,14 +218,16 @@ class InvitationViewSet(viewsets.ModelViewSet):
     #     url_path=CREATE_AND_SEND_URL
     # )
     @action(
-        methods=['post'],detail=False
+        methods=['post'], detail=False
     )
     def send(self, request):
         serializer = self.get_serializer(data=request.data)
         serializer.is_valid(raise_exception=True)
         email = serializer.data['email']
-        authorentity = AuthorModel.objects.get(pk=serializer.data['authorentity'])
-        invitation = InvitationModel.create(email=email,authorentity=authorentity, inviter=request.user)
+        authorentity = AuthorModel.objects.get(
+            pk=serializer.data['authorentity'])
+        invitation = InvitationModel.create(
+            email=email, authorentity=authorentity, inviter=request.user)
         self._prepare_and_send(invitation, request)
         content = {'detail': 'Invite sent'}
         return Response(content, status=status.HTTP_200_OK)
@@ -235,7 +243,8 @@ class InvitationViewSet(viewsets.ModelViewSet):
         serializer = self.get_serializer(data=request.data)
         serializer.is_valid(raise_exception=True)
         inviter = request.user
-        authorentity = AuthorModel.objects.get(pk=serializer.data['authorentity'])
+        authorentity = AuthorModel.objects.get(
+            pk=serializer.data['authorentity'])
         for email in serializer.data['email']:
             invitation = InvitationModel.create(
                 email=email, authorentity=authorentity, inviter=inviter)
@@ -244,7 +253,26 @@ class InvitationViewSet(viewsets.ModelViewSet):
         return Response(content, status=status.HTTP_200_OK)
 
 
-@api_view(('POST', 'GET'))
+INVALID_INVITE = 'invalid-invite'
+ACCEPTED_INVITE = 'accepted-invite'
+EXPIRED_INITE = 'expired-invite'
+SUCCESS_INVITE = 'success'
+
+
+def verify_invitation_object(invitation):
+    if not invitation:
+        return {INVALID_INVITE: 'Invalid/Expired Invitation.'}
+
+    if invitation.accepted:
+        return {ACCEPTED_INVITE: 'This invitation has already been accepted.'}
+
+    if invitation.key_expired():
+        return {EXPIRED_INITE: 'Invitation has expired.'}
+
+    return {SUCCESS_INVITE: 'This is a valid invitation'}
+
+
+@api_view(['GET'])
 @permission_classes((AllowAny,))
 def accept_invitation(request, key):
     def get_object():
@@ -255,28 +283,58 @@ def accept_invitation(request, key):
 
     invitation = get_object()
 
-    login_data = {
-        # 'LOGIN_REDIRECT': invitations_settings.LOGIN_REDIRECT
-    }
-    signup_data = {
-        # 'SIGNUP_REDIRECT': invitations_settings.SIGNUP_REDIRECT
-    }
+    verified_data = verify_invitation_object(invitation)
 
-    if invitations_settings.GONE_ON_ACCEPT_ERROR and \
-        (not invitation or
-         (invitation and (invitation.accepted or
-                          invitation.key_expired()))):
-        return Response(status=status.HTTP_410_GONE)
+    return Response(
+        verified_data,
+        status=status.HTTP_200_OK
+    )
 
-    if not invitation:
+
+@api_view(['POST'])
+@permission_classes((AllowAny,))
+def signup(request, key):
+    def get_object():
+        try:
+            return InvitationModel.objects.get(key=key.lower())
+        except InvitationModel.DoesNotExist:
+            return None
+
+    invitation = get_object()
+    INVITE_STATUS = verify_invitation_object(invitation)
+    if SUCCESS_INVITE in INVITE_STATUS:
+        if invitation.authorentity.guestlogin is None:
+            request.data['group_id'] = "-1"
+            serialized = CUserSerializer(data=request.data)
+            serialized.is_valid(raise_exception=True)
+            user = serialized.save()
+            invitation.authorentity.guestlogin = user
+            invitation.authorentity.save()
+            invitation.accepted = True
+            invitation.save()
+            invite_accepted.send(sender=None, email=invitation.email)
+            get_invitations_adapter().add_message(
+                request,
+                messages.SUCCESS,
+                'invitations/messages/invite_accepted.txt',
+                {
+                    'email': invitation.email,
+                    'author': invitation.authorentity.pk
+                }
+            )
+        else:
+            return Response(
+                {INVALID_INVITE: AuthorModel._meta.verbose_name +
+                    ' has already been assigned a Guest Account.'},
+                status=status.HTTP_200_OK
+            )
+    elif INVALID_INVITE in INVITE_STATUS:
         get_invitations_adapter().add_message(
             request,
             messages.ERROR,
             'invitations/messages/invite_invalid.txt'
         )
-        return Response(login_data, status=status.HTTP_200_OK)
-
-    if invitation.accepted:
+    elif ACCEPTED_INVITE in INVITE_STATUS:
         get_invitations_adapter().add_message(
             request,
             messages.ERROR,
@@ -285,9 +343,7 @@ def accept_invitation(request, key):
                 'email': invitation.email
             }
         )
-        return Response(login_data, status=status.HTTP_200_OK)
-
-    if invitation.key_expired():
+    elif EXPIRED_INITE in INVITE_STATUS:
         get_invitations_adapter().add_message(
             request,
             messages.ERROR,
@@ -296,28 +352,7 @@ def accept_invitation(request, key):
                 'email': invitation.email
             }
         )
-        return Response(signup_data, status=status.HTTP_200_OK)
-
-    if not invitations_settings.ACCEPT_INVITE_AFTER_SIGNUP:
-        invitation.accepted = True
-        invitation.save()
-        invite_accepted.send(sender=None, email=invitation.email)
-        get_invitations_adapter().add_message(
-            request,
-            messages.SUCCESS,
-            'invitations/messages/invite_accepted.txt',
-            {
-                'email': invitation.email,
-                'author':invitation.authorentity.pk
-            }
-        )
-        signup_data.update(
-            {
-                'account_verified_email': invitation.email,
-                'account_verified_author': invitation.authorentity.pk
-            }
-        )
     return Response(
-        signup_data,
+        INVITE_STATUS,
         status=status.HTTP_200_OK
     )
